package pl.labno.bernard.seamTestInAction;

import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.security.Restrict;
import org.jboss.seam.framework.EntityHome;
import org.jboss.seam.framework.EntityNotFoundException;

@Name("tagHome")
public class TagHome extends EntityHome<Tag> {

    private boolean editMode;

    @Restrict("#{s:hasRole('ADMIN')}")
    @Override
    public String persist() {
        setEditMode(false);
        return super.persist();
    }

    @Restrict("#{s:hasRole('ADMIN')}")
    @Override
    public String update() {
        setEditMode(false);
        return super.update();
    }

    @Restrict("#{s:hasRole('ADMIN')}")
    @Override
    public String remove() {
        return super.remove();
    }

    public boolean isEditMode() {
        return editMode;
    }

    public void setEditMode(boolean editMode) {
        this.editMode = editMode;
    }

    public void edit() {
        setEditMode(true);
    }

    /**
     * Invoke this method to switch to editMode.
     */
    public void startCreateProcess() {
        setEditMode(true);
        clearInstance();
    }

    /**
     * Used in tag.page.xml to see if identifier is valid.
     * Identifier may be null only if user wants to add new tag
     * so this component must be in edit mode.
     */
    public void validate() {
        if (getId() == null && !isEditMode()) {
            throw new EntityNotFoundException(getId(), Tag.class);
        }
        getInstance();
    }

}
