package pl.labno.bernard.seamTestInAction.test;

import org.jboss.seam.contexts.Contexts;
import org.jboss.seam.mock.SeamTest;

import javax.persistence.EntityManager;

public class ExtendedSeamTest extends SeamTest {

    /**
     * Remember to invoke super.afterRequest if you override it.
     */
    public class RenderedNonFacesRequest extends NonFacesRequest {

        public RenderedNonFacesRequest() {
        }

        /**
         * @param viewId the view id of the form that was submitted
         */
        public RenderedNonFacesRequest(String viewId) {
            super(viewId);
        }

        /**
         * @param viewId         the view id of the form that was submitted
         * @param conversationId the conversation id
         */
        public RenderedNonFacesRequest(String viewId, String conversationId) {
            super(viewId, conversationId);
        }

        @Override
        protected void afterRequest() {
            assert isRenderResponseBegun();
        }
    }

    /**
     * Remember to invoke super.afterRequest if you override it.
     */
    public class InvokedApplicationFacesRequest extends FacesRequest {

        public InvokedApplicationFacesRequest() {
        }

        /**
         * @param viewId the view id of the form that was submitted
         */
        public InvokedApplicationFacesRequest(String viewId) {
            super(viewId);
        }

        /**
         * @param viewId         the view id of the form that was submitted
         * @param conversationId the conversation id
         */
        public InvokedApplicationFacesRequest(String viewId, String conversationId) {
            super(viewId, conversationId);
        }

        @Override
        protected void afterRequest() {
            assert isInvokeApplicationBegun();
        }
    }

    protected EntityManager getEntityManager() {
        return (EntityManager) getInstance(Config.ENTITY_MANAGER_NAME);
    }

    protected void login(final String username, final String password) throws Exception {
        new InvokedApplicationFacesRequest(Config.LOGIN_VIEW) {
            @Override
            protected void updateModelValues() throws Exception {
                setValue("#{credentials.username}", username);
                setValue("#{credentials.password}", password);
            }

            @Override
            protected void invokeApplication() throws Exception {
                invokeAction("#{identity.login}");
                assert (Boolean) invokeAction("#{identity.isLoggedIn}");
            }
        }.run();
    }

    protected void loginAsAdmin() throws Exception {
        login(Config.ADMIN_USERNAME, Config.ADMIN_PASSWORD);
    }

    protected void loginAsMember() throws Exception {
        login(Config.MEMBER_USERNAME, Config.MEMBER_PASSWORD);
    }

    protected void logout() throws Exception {
        new InvokedApplicationFacesRequest(Config.LOGIN_VIEW) {
            @Override
            protected void invokeApplication() throws Exception {
                invokeAction("#{identity.logout}");
                assert !(Boolean) invokeAction("#{identity.isLoggedIn}");
            }
        }.run();
    }

    protected Object getHandledException() {
        return Contexts.lookupInStatefulContexts("org.jboss.seam.handledException");
    }

    public static class Holder<T> {
        private T value;

        public T getValue() {
            return value;
        }

        public void setValue(T value) {
            this.value = value;
        }
    }
}
