package pl.labno.bernard.seamTestInAction.test;

import org.jboss.seam.Component;
import org.jboss.seam.framework.EntityNotFoundException;
import org.jboss.seam.mock.AbstractSeamTest;
import org.jboss.seam.security.AuthorizationException;
import org.jboss.seam.security.NotLoggedInException;
import org.testng.annotations.DataProvider;
import org.testng.annotations.Test;
import pl.labno.bernard.seamTestInAction.Tag;
import pl.labno.bernard.seamTestInAction.TagHome;
import static pl.labno.bernard.seamTestInAction.test.Config.*;

import javax.persistence.EntityManager;
import java.util.List;

public class TagCRUDTest extends ExtendedSeamTest {

    public static Long id;
    private String modifiedName = "Modified tag";
    private String modifiedName2 = "Modified2 tag";
    private static final String INVALID_INPUT_PROVIDER = "createInvalidInput";

    @DataProvider(name = INVALID_INPUT_PROVIDER)
    public Object[][] createInvalidInput() {
        return new Object[][]{
                new Object[]{""},
                new Object[]{null},
                new Object[]{"asdfghjkl asdfghjkl a"}
        };
    }
    
    @Test(dataProvider = INVALID_INPUT_PROVIDER)
    public void invalidInputTest(final String name) throws Exception {
        loginAsAdmin();
        String conversationId = new RenderedNonFacesRequest(TAG_VIEW) {
            @Override
            protected void beforeRequest() {
                setParameter("actionMethod", TAG_LIST_VIEW + ":tagHome.startCreateProcess");
            }

            @Override
            protected void renderResponse() throws Exception {
                assert (Boolean) getValue("#{tagHome.editMode}");
            }
        }.run();
        new FacesRequest(TAG_VIEW, conversationId) {

            @Override
            protected void applyRequestValues() throws Exception {
                setValue("#{tagHome.instance.name}", name);
            }

            @Override
            protected void processValidations() throws Exception {
                validateValue("#{tagHome.instance.name}", name);
            }

            @Override
            protected void afterRequest() {
                assert isValidationFailure();
                assert !isInvokeApplicationBegun();
            }
        }.run();
    }

    @Test
    public void createTag() throws Exception {
        loginAsAdmin();
        String conversationId = new RenderedNonFacesRequest(TAG_VIEW) {
            @Override
            protected void beforeRequest() {
                setParameter("actionMethod", TAG_LIST_VIEW + ":tagHome.startCreateProcess");
            }

            @Override
            protected void renderResponse() throws Exception {
                assert (Boolean) getValue("#{tagHome.editMode}");
            }
        }.run();

        new InvokedApplicationFacesRequest(TAG_VIEW, conversationId) {
            @Override
            protected void processValidations() throws Exception {
                validateValue("#{tagHome.instance.name}", "My new legal act");
                assert !isValidationFailure();
            }

            @Override
            protected void updateModelValues() throws Exception {
                setValue("#{tagHome.instance.name}", "My new legal act");
            }

            @Override
            protected void invokeApplication() throws Exception {
                invokeAction("#{tagHome.persist}");
                id = (Long) getValue("#{tagHome.instance.id}");
                assert null != id;
            }
        }.run();
    }

    @Test(dependsOnMethods = {"createTag"}, groups = TAG_DEPENDANT_TESTS)
    public void editTag() throws Exception {
        loginAsAdmin();
        String conversationId = enterTagView(id);
        conversationId = new InvokedApplicationFacesRequest(TAG_VIEW, conversationId) {
            @Override
            protected void invokeApplication() throws Exception {
                invokeAction("#{tagHome.edit}");
                assert (Boolean) getValue("#{tagHome.editMode}");
            }
        }.run();
        new InvokedApplicationFacesRequest(TAG_VIEW, conversationId) {
            @Override
            protected void updateModelValues() throws Exception {
                setValue("#{tagHome.instance.name}", modifiedName);
            }

            @Override
            protected void invokeApplication() throws Exception {
                invokeAction("#{tagHome.update}");
            }
        }.run();

        new RenderedNonFacesRequest(TAG_LIST_VIEW) {
            @Override
            protected void renderResponse() throws Exception {
                Tag tag = (Tag) ((EntityManager) getInstance("entityManager")).createQuery("from Tag where id=?").setParameter(1, id).getSingleResult();
                assert modifiedName.equals(tag.getName());
            }
        }.run();
    }

    @Test(dependsOnMethods = {"createTag"}, groups = TAG_DEPENDANT_TESTS)
    public void cancelEditTag() throws Exception {
        loginAsAdmin();
        final Holder<String> nameBeforModification = new Holder<String>();
        String conversationId = enterTagView(id);
        conversationId = new InvokedApplicationFacesRequest(TAG_VIEW, conversationId) {
            @Override
            protected void invokeApplication() throws Exception {
                nameBeforModification.setValue((String) getValue("#{tagHome.instance.name}"));
                invokeAction("#{tagHome.edit}");
                assert (Boolean) getValue("#{tagHome.editMode}");
            }
        }.run();
        new InvokedApplicationFacesRequest(TAG_VIEW, conversationId) {
            @Override
            protected void updateModelValues() throws Exception {
                setValue("#{tagHome.instance.name}", modifiedName2);
            }

        }.run();
        new InvokedApplicationFacesRequest(TAG_VIEW, conversationId) {
            @Override
            protected void invokeApplication() throws Exception {
                invokeAction("cancel");
            }

        }.run();

        new RenderedNonFacesRequest(TAG_VIEW) {
            @Override
            protected void beforeRequest() {
                setParameter("id", id.toString());
            }

            @Override
            protected void renderResponse() throws Exception {
                assert nameBeforModification.getValue().equals(getValue("#{tagHome.instance.name}"));
            }
        }.run();
    }

    @Test(dependsOnMethods = "createTag", groups = TAG_DEPENDANT_TESTS)
    public void enterTagViewAsAdmin() throws Throwable {
        loginAsAdmin();
        new RenderedNonFacesRequest(TAG_LIST_VIEW) {
            @Override
            protected void renderResponse() {
                assert (Boolean) getValue("#{tagList.resultCount > 0}");
                @SuppressWarnings("unchecked")
                List<Tag> tags = (List<Tag>) getValue("#{tagList.resultList}");
                boolean tagExists = false;
                for (Tag tag : tags) {
                    if (id.equals(tag.getId())) {
                        tagExists = true;
                    }
                }
                assert tagExists;
            }
        }.run();
    }

    @Test
    public void enterNonExistingTagView() throws Throwable {
        loginAsAdmin();
        new RenderedNonFacesRequest(TAG_VIEW) {
            @Override
            protected void beforeRequest() {
                setParameter("id", "-1");
            }

            @Override
            protected void renderResponse() {
                assert getHandledException() instanceof EntityNotFoundException;
            }
        }.run();
        new RenderedNonFacesRequest(TAG_VIEW) {
            @Override
            protected void renderResponse() {
                assert getHandledException() instanceof EntityNotFoundException;
            }
        }.run();
    }

    @Test(dependsOnMethods = "createTag", groups = TAG_DEPENDANT_TESTS)
    public void enterTagViewAsNonAdmin() throws Throwable {
        new NonFacesRequest(TAG_VIEW) {
            @Override
            protected void beforeRequest() {
                setParameter("id", id.toString());
            }

            @Override
            protected void afterRequest() {
                assert !isRenderResponseBegun();
            }
        }.run();
        loginAsMember();
        new RenderedNonFacesRequest(TAG_VIEW) {
            @Override
            protected void beforeRequest() {
                setParameter("id", id.toString());
            }

            @Override
            protected void renderResponse() {
                assert getHandledException() instanceof AuthorizationException;
            }
        }.run();
    }

    @Test(dependsOnGroups = TAG_DEPENDANT_TESTS)
    public void removeTag() throws Exception {
        loginAsAdmin();
        String conversationId = enterTagView(id);
        new InvokedApplicationFacesRequest(TAG_VIEW, conversationId) {
            @Override
            protected void invokeApplication() throws Exception {
                invokeAction("#{tagHome.remove}");
            }
        }.run();
        new RenderedNonFacesRequest(TAG_LIST_VIEW) {

            @Override
            protected void renderResponse() throws Exception {
                assert null == getEntityManager().find(Tag.class, id);
            }

        }.run();
    }

    @Test
    public void componentSecurity() throws Throwable {
        new ComponentTest() {
            protected void testComponents() throws Exception {
                try {
                    invokeMethod("#{tagHome.persist}");
                    assert false;
                } catch (Exception e) {
                    assert e.getCause() instanceof NotLoggedInException;
                }
                try {
                    invokeMethod("#{tagHome.update}");
                    assert false;
                } catch (Exception e) {
                    assert e.getCause() instanceof NotLoggedInException;
                }
                try {
                    invokeMethod("#{tagHome.remove}");
                    assert false;
                } catch (Exception e) {
                    assert e.getCause() instanceof NotLoggedInException;
                }
            }
        }.run();
        loginAsMember();
        new ComponentTest() {
            protected void testComponents() throws Exception {
                try {
                    invokeMethod("#{tagHome.persist}");
                    assert false;
                } catch (Exception e) {
                    assert e.getCause() instanceof AuthorizationException;
                }
                try {
                    invokeMethod("#{tagHome.update}");
                    assert false;
                } catch (Exception e) {
                    assert e.getCause() instanceof AuthorizationException;
                }
                try {
                    invokeMethod("#{tagHome.remove}");
                    assert false;
                } catch (Exception e) {
                    assert e.getCause() instanceof AuthorizationException;
                }
            }
        }.run();
    }

    @Test
    public void enterTagListViewAsNonAdmin() throws Exception {
        new NonFacesRequest(TAG_LIST_VIEW) {
            @Override
            protected void afterRequest() {
                assert !isRenderResponseBegun();
            }
        }.run();
        loginAsMember();
        new RenderedNonFacesRequest(TAG_LIST_VIEW) {
            @Override
            protected void renderResponse() throws Exception {
                assert getHandledException() instanceof AuthorizationException;
            }
        }.run();
    }


    /**
     * NonFacesRequest simulating entering page with Tag's details.
     *
     * @param tagId id of Tag
     * @return conversation id
     * @throws Exception unspecified
     */
    private String enterTagView(final Long tagId) throws Exception {
        return new RenderedNonFacesRequest(TAG_VIEW) {

            @Override
            protected void beforeRequest() {
                setParameter("id", tagId.toString());
            }

            @Override
            protected void renderResponse() throws Exception {
                TagHome home = (TagHome) Component.getInstance(TagHome.class);
                assert tagId != null && tagId.equals(home.getId()) && home.isManaged();
            }

        }.run();
    }
}
